import json
import requests
import logging
import sys

__author__ = 'Sisense'

# Logging settings
LOG_FILE_NAME = 'C:\ProgramData\Sisense\PrismServer\example.log'
LOG_LEVEL = logging.DEBUG

# create logger
logger = logging.getLogger('updateShares.py')
logger.setLevel(logging.DEBUG)

# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# create file handler and set level to debug
fh = logging.FileHandler(LOG_FILE_NAME)
fh.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# add formatter to ch and fh
ch.setFormatter(formatter)
fh.setFormatter(formatter)

# add ch to logger
logger.addHandler(ch)

# add fh to logger
logger.addHandler(fh)

REST_API_TOKEN = 'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjoiNWM0YTFhMjk4ZTFmY2FkYjFiNjYwNzVmIiwiYXBpU2VjcmV0IjoiNzdlMzBkNzEtYjlmYy1hMDE5LTI2MzktN2JjYTIxOGU1NTkwIiwiaWF0IjoxNTUwNjkyNzkwfQ.uuk3WhM7FtRLYdgbToks_lbr3iMd-_5Grw_ITmVdJEs'
SISENSE_BASE_URL = "http://localhost:8081"

def call_api(http_method, url_suffix, params=None, payload=None, sys_admin=False):

    headers = {'Authorization': "Bearer " + REST_API_TOKEN}

    if payload:
        headers['Content-Type'] = 'application/json'

    url = SISENSE_BASE_URL + url_suffix

    response = None
    data = None

    if http_method == 'GET':
        response = requests.get(url, params=params, headers=headers)
    elif http_method == 'POST':
        data = json.dumps(payload)
        response = requests.post(
            url, data=data, params=params, headers=headers)
    elif http_method == 'PATCH':
        data = json.dumps(payload)
        response = requests.patch(
            url, data=data, params=params, headers=headers)

    return response


def get_users(params=None):

    res = call_api('GET', '/api/users', params=params)
    if res.reason == 'OK':
        return json.loads(res.text)

    return None


def get_shares(dashboard_id):

    print ('/api/v1/dashboards/{0}/shares'.format(dashboard_id))

    res = call_api('GET', '/api/v1/dashboards/{0}/shares'.format(dashboard_id))

    if res.reason == 'OK':
        return json.loads(res.text)

    return None

def get_subscription(dashboard_id):

    res = call_api('GET', '/api/shares/dashboard/{0}'.format(dashboard_id))

    if res.reason == 'OK':
        return json.loads(res.text)['subscription']

    return None

# ----------------------
# Share a dashboard with a user
# ----------------------


def share_dashboard_with_user(user_name, dashboard_id, role='view', subscribe=False):

    user_id = get_user_id(user_name)

    logger.debug("User id retrieved: " + user_id)

    if user_id is not None:
        share_dashboard(user_id, dashboard_id, 'user', role, subscribe)

# ----------------------
#  Share a dashboard with a group
# -----------------------


def share_dashboard_with_group(group_name, dashboard_id, role='view', subscribe=False):

     # Get the groupId required for updating the share
    group_id = get_group_id(group_name)

    logger.debug("Group id retrieved: " + group_id)

    if group_id is not None:
        share_dashboard(group_id, dashboard_id, 'group', role, subscribe)

# --------------------
# Generic share dashborad function to be called by
# share_dashboard_with_user and share_dashboard_with_group
# functions.
# --------------------

def share_dashboard(share_id, dashboard_id, share_type, role='view', subscribe=False):

    logger.info("Stated share_dahsboard , sharing dashboard " +
                 dashboard_id + " with " + share_type + " " + share_id)

    # Retrieve the current shares
    curr_shares = get_shares(dashboard_id)

    logger.debug("Current shares:\n" +
                  str(json.dumps(curr_shares, indent=4, separators=(',', ': '))))

    # Get current subscription
    curr_subsc = get_subscription(dashboard_id)

    logger.debug("Current subrscription:\n" +
                  str(json.dumps(curr_subsc, indent=4, separators=(',', ': '))))

    # Check share doesn't already exist
    curr_shares = [
        shares for shares in curr_shares if shares.get('shareId') != share_id]

    # Add the new share to the shares array
    curr_shares.append({
        "shareId": share_id,
        "type": share_type,
        "rule": role,
        "subscribe": subscribe
    })

    # Build the payload JSON
    payload = {"sharesTo": curr_shares}
    payload['subscription'] = curr_subsc

    logger.debug("Current shares after append:\n" +
                  str(json.dumps(curr_shares, indent=4, separators=(',', ': '))))

    # Run the command
    res = call_api('POST', '/api/shares/dashboard/' +
                   dashboard_id, payload=payload)

    logger.debug(res.text)

    if res.reason == 'OK':
        return '200'

    return None


# -------------------
# Share all dashbaords in a folder with a user
# This function simply loops over the dahsboards in
# a folder and shares each dashbpoard with a user
# -------------------


def share_folder_with_user(email, folder_name):

    logger.info("Stated share_folder_with_user , sharing folder " +
                 folder_name + " with user " + email)

    dashboard_ids_arr = get_dashboards_in_folder(folder_name)

    logger.debug("Found dashboards in folder:\n" + str(dashboard_ids_arr))

    for dashboard_id in dashboard_ids_arr:

        res = share_dashboard_with_user(email, dashboard_id)

        print ("Sharing dashboard: " + dashboard_id +
               " with " + email + ". Result: " + str(res))

        logger.info(__name__ + "complete")


def share_folder_with_group(group_name, folder_name):
    logger.info("Stated share_folder_with_group , sharing folder " +
                 folder_name + " with group " + group_name)

    dashboard_ids_arr = get_dashboards_in_folder(folder_name)

    logger.debug("Found dashboards in folder:\n" + str(dashboard_ids_arr))

    for dashboard_id in dashboard_ids_arr:

        res = share_dashboard_with_group(group_name, dashboard_id)

        print ("Sharing dashboard: " + dashboard_id +
               " with " + group_name + ". Result: " + str(res))

        logger.info(__name__ + "complete")


def get_group_id(group_name):

    res = call_api('GET', '/api/v1/groups?name=' + group_name)

    if res.status_code == 404 and res.reason == 'Not Found':
        return None

    elif res.status_code == 200:

        group_json = json.loads(res.text)
        group_json = group_json[0]  # since the GET call returns an array

        return (group_json.get('_id'))
    else:
        return None


def get_user_id(user_name):

    res = call_api('GET', '/api/users/' + user_name)

    if res.status_code == 404 and res.reason == 'Not Found':

        logger.info("User name: " + user_name + " not found.")
        return (res.status_code)

    elif res.status_code == 200:

        logger.info("Found user name: " + user_name)
        user_json = json.loads(res.text)

        logger.debug("User_json:" + str(user_json))

        return (user_json.get('_id'))
    else:
        return None

# ------------------
# Get folder ID by name
# ------------------


def get_folder_id(folder_name):

    res = call_api('GET', '/api/v1/folders?name=' + folder_name)

    if res.status_code == 404 and res.reason == 'Not Found':
        return (None)
    elif res.status_code == 200:

        folder_json = json.loads(res.text)
        folder_json = folder_json[0]  # since the GET call returns an array
        logger.debug('Folder found. Folderid=' + folder_json.get('oid'))

        return (folder_json.get('oid'))
    else:
        return (None)

#-----------------
# Find ids of dashboards in a folder, by folder name
#-----------------


def get_dashboards_in_folder(folder_name):

    folder_id = get_folder_id(folder_name)

    res = call_api('GET', '/api/v1/dashboards?parentFolder=' +
                   folder_id + '&fields=_id')

    if res.status_code == 200:
        logger.debug("Found dashboards for folder: " + res.text)

        dashboard_ids = []
        dashboard_ids = [ids['oid'] for ids in json.loads(res.text)]

        logger.debug("Dashboard Ids found in folder: " + str(dashboard_ids))

    return dashboard_ids


def main(argv):

    try:

        # share_dashboard_with_user('deddyhogeg@gmail.com', '5c5dc3b8693225434cdf489b')
        # share_dashboard_with_group('Test', '56f1aa1dbc4b62043a000028')
        # share_folder_with_user("deddyhogeg@gmail.com", "blah")
        share_folder_with_group("TEST_GROUP","blah")
        # get_folder_id("JS")
        # get_dashboards_in_folder("JS")

    except Exception as e:

        logger.error(e)

    else:

        logger.info("Completed Successfully")

if __name__ == '__main__':
    main(sys.argv)
